import * as Sortable from 'sortablejs';
import * as _ from 'lodash';
import eventbus from '../../service/eventbus';
// import eventbus from '../../../views/editor/service/eventbus';
import Service from "../../service/tool-service";

let common = {
    createNewInstanceKey () {
        return 'gaea_instance_'+Number(Math.random().toString().substr(3,3) + Date.now()).toString(36);
    }
};

const viewport = {
    namespaced: true,
    state: {
        localViewportDOM: null,
        localInstanceDoms: new Map(),
        localElement: "",
        localInstances: new Map(),
        viewportDOM: null,
        rootInstanceKey: null,
        instances: new Map(),
        instanceDoms: new Map(),
        currentDragInfo: null,
        currentHoverInstanceKey: null,
        currentEditInstanceKey: null,
        currentFormInstanceKey: null,
        currentInfoCellInstanceKey: null,
        currentBoxInstanceKey: null,
        parentEditInstanceKey: null,
        dragStartDataReady: false,
        isDrag:false,
        pagePriviewData:{},
        /**
         * 跨父級拖动，保存当前的那个对象
         */
        saveDragObj: {},
        /**
         * 鼠标离开视图区域
         */
        mouseLeaveViewport: 'mouseLeaveViewport',
        /**
         * 鼠标移动到某个组件上
         */
        mouseHoveringComponent: 'mouseHoveringComponent',
        /**
         * 鼠标移动到面包屑
         */
        mouseHoveringCrumbItem: 'mouseHoveringCrumbItem',
        /**
         * 鼠标移动到面包屑
         */
        mouseLeaveCrumbItem: 'mouseLeaveCrumbItem',
        /**
         * 视图区域发生更新
         */
        viewportUpdated: 'viewportUpdated',
        /**
         * 刷新某个实例
         */
        instanceUpdate: 'instanceUpdate',
        /**
         * 页面冲渲染
         */
        refreshPage: 'refreshPage',
        /**
         * 触发编辑器回调
         */
        emitEditorCallback: 'emitEditorCallback',
        /**
         * 当前点击选组件改变
         */
        currentEditInstanceChange: 'currentEditInstanceChange'
    },
    mutations: {
        startDrag (state, dragInfo) {
            state.currentDragInfo = dragInfo;
            state.isDrag=true;
        },
        endDrag (state) {
            state.currentDragInfo = null;
            state.isDrag=false;
        },
        setViewportDOM (state, viewportDOM) {
            state.viewportDOM = viewportDOM;
        },
        initViewport (state) {
            this.commit('viewport/addInstance', {
                gaeaKey: 'gaea-container',
                parentInstanceKey: null,
                indexPosition: null
            });
            let rootInstance = Array.from(state.instances);
            const rootInstanceKey = rootInstance[0][0];
            this.commit('viewport/setRootInstanceKey', rootInstanceKey);
        },
        //note 当前拖动的实例和目标实例
        dealCurLevelAndTargetLevel(state,obj){
            //
            debugger;
            let instance = state
                .instances
                .get(obj.currentInstance);
            let parentInstance= state
                .instances
                .get(obj.targetInstance);
            //note 如果已经是根元素, 直接返回空数组,获取父级的level和当前的level
            let parentLevel=parentInstance.vm.level;
            if(!parentLevel){
                return;
            }
            if(instance.gaeaKey=="gaea-cell-table"){
                return;
            }
            //note 标示是当前拖动进来的组件   在这里判断当前的是否大于父级的level，如果大于正常，否则删除拖动进来的组件，并提示，不符合规则
            let currLevel=instance.vm.level;
            if(currLevel<=parentLevel){
                let data={
                    title:"提示",
                    content:"拖拽不符合规则"
                }
                state.currentEditInstanceKey=obj.currentInstance;
                eventbus.$emit("openModal",data);
                //note 获取存储的实例
                console.log(state.saveDragObj);
                setTimeout(()=>{
                    this.commit('viewport/deleteData', state.currentEditInstanceKey);
                    instance.key=obj.currentInstance;
                    eventbus.$emit("addMoveInstance",state.saveDragObj);
                },800);
            }
            return;
        },
        dealLevel(state){
            let instance = state
                .instances
                .get(state.currentEditInstanceKey);
            let parentInstance= state
                .instances
                .get(state.parentEditInstanceKey);
            //note 如果已经是根元素, 直接返回空数组,获取父级的level和当前的level
            if(!instance||instance.parentInstanceKey== "gaea_instance_1"){
                return;
            }
            let parentLevel=parentInstance.vm.level;
            if(!parentLevel){
                return;
            }
            //note 标示是当前拖动进来的组件   在这里判断当前的是否大于父级的level，如果大于正常，否则删除拖动进来的组件，并提示，不符合规则
            let currLevel=instance.vm.level;
            if(instance.gaeaKey=="gaea-cell-table"||parentInstance.gaeaKey=="gaea-info-base"&&instance.gaeaKey=="gaea-cell"){

            }else if(currLevel<=parentLevel){
                let data={
                    title:"提示",
                    content:"拖拽不符合规则"
                }
                eventbus.$emit("openModal",data);
                setTimeout(()=>{
                    this.commit('viewport/deleteData', state.currentEditInstanceKey);
                },800);
            }
            return;
        },
        addInstance (state, params) {
            let newInstanceKey = "";
            if(params.gaeaKey=="gaea-container"){
                newInstanceKey = 'gaea_instance_1';
            }else{
                newInstanceKey = params.newInstanceKey||common.createNewInstanceKey();
            }
            state
                .instances
                .set(newInstanceKey, {
                    gaeaKey: params.gaeaKey,
                    data: params.data||{
                        props: null
                    },
                    slots: {},
                    parentInstanceKey: params.parentInstanceKey,
                    preGaeaKey: params.preGaeaKey,
                    vm: null
                });
            // 先执行 callback  挂载 vm 再执行 修改属性样式
            //note 更新实例数组顺序，让新添加进来的数组放在父数组下面
            debugger;
            let instanceArr=[...state.instances];
            let parentIndex=0;
            try {
                state.instances.forEach(function(v,k) {
                    console.log(v); //这样就会分别将name遍历出来
                    parentIndex++;
                    if(k==params.parentInstanceKey){
                        //note 说明得到父元素
                        throw new Error("EndIterative");
                    }
                });
            } catch(e) {
                //note 只有在捕获到父元素的情况下且是处于拖动的情况下，才会挪动数组
                if(state.isDrag){
                    state.instances=[...state.instances];
                    let index=state.instances.length-1;
                    //note 修改数组位置
                    if(index>parentIndex){
                        let strInstance=[];
                        let myTest=state.instances;
                        //    strInstance=myTest.concat();
                        for(let key=0;key<myTest.length;key++){
                            let myArr=[];
                            myArr[0]=myTest[key][0];
                            let myObj={};
                            myObj.parentInstanceKey=myTest[key][1].parentInstanceKey;
                            myObj.slots=myTest[key][1].slots;
                            myArr[1]=myObj;
                            strInstance.push(myArr);
                            // val.data.property.value=val.data.property.label;
                        }
                        //note 找到当前父组件名称 在所有组件中的数量
                        let jsonInstance=JSON.stringify(strInstance);
                        let parentLength=((jsonInstance.split(params.parentInstanceKey)).length-1)-2;
                        //note 父元素中的数量，在所有的拖拽中，拖拽的元素永远放在父元素里的最底下一个

                        state.instances.splice(parentIndex,0,state.instances[index]);
                        state.instances.splice(index+1,1)

                    }
                    else{
                        //如果当前元素在拖动目标位置的上方，先将当前元素从数组拿出，数组长度-1，我们直接给数组拖动目标位置+1的地方新增一个和当前元素值一样的元素，
                        //这时，数组len不变，我们再把数组之前的那个拖动的元素删除掉，下标还是index
                        state.instances.splice(parentIndex+1,0,state.instances[index]);
                        state.instances.splice(index,1)
                    }
                }
                if(e.message!="EndIterative") throw e;
            };

            state.instances = new Map(state.instances);
            if (params.cb) {
                params
                    .cb
                    .call(null, newInstanceKey);
            }
            if(params.gaeaKey=="gaea-form"){
                state.currentFormInstanceKey=newInstanceKey;
            }
            if(params.gaeaKey=="gaea-box"){
                state.currentBoxInstanceKey=newInstanceKey;
            }

            // 然后再执行修改样式
            if (params.parentInstanceKey !== null) {
                const parentInstance = state
                    .instances
                    .get(params.parentInstanceKey);
                if (!parentInstance.slots[params.slotName]) {
                    parentInstance.slots[params.slotName] = [];
                }
                parentInstance
                    .slots[params.slotName]
                    .splice(params.indexPosition, 0, newInstanceKey);
                // 如果父级和自身都是 gaea-container，且父级是 display:flex，那么子元素默认 flexDirection 与父级元素相反

                //note 处理业务逻辑
                state.currentEditInstanceKey=newInstanceKey;
                state.parentEditInstanceKey= params.parentInstanceKey;
                if (parentInstance.gaeaKey === 'gaea-container' && params.gaeaKey === 'gaea-container') {
                    //     let crrtInstance = state
                    //         .instances
                    //         .get(newInstanceKey);
                    //     if (parentInstance.vm.$el.style.display === 'flex' && crrtInstance.vm.$el.style.display === 'flex') {
                    //         switch (parentInstance.vm.$el.style.flexDirection) {
                    //             case 'column':
                    //                 this.commit('viewport/setInstanceProps', {
                    //                     vm: crrtInstance.vm,
                    //                     key: 'styles',
                    //                     value: {
                    //                         flexDirection: 'row'
                    //                     }
                    //                 });
                    //                 break;
                    //             case 'row':
                    //             default:
                    //                 this.commit('viewport/setInstanceProps', {
                    //                     vm: crrtInstance.vm,
                    //                     key: 'styles',
                    //                     value: {
                    //                         flexDirection: 'column'
                    //                     }
                    //                 });
                    //                 break;
                    //         }
                    //     }
                }
            }
        },
        setInstanceProps (state, {vm, key, value}) {
            if (Object.prototype.toString.call(value) === '[object Object]') {
                let obj = _.get(vm.$data, key);
                let newObj=Object.assign(obj, value)
                _.set(vm.$data, key, newObj);
            } else {
                _.set(vm.$data, key, value);
            }
            vm.$forceUpdate();
        },
        setRootInstanceKey (state, key) {
            state.rootInstanceKey = key;
        },
        setDomInstance (state, {key, instance}) {
            state
                .instanceDoms
                .set(key, instance);
        },
        setCurrentHoverInstanceKey (state, instanceKey) {
            state.currentHoverInstanceKey = instanceKey;
        },
        moveInstance (state, {sourceTargetKey, targetParentKey, targetIndex, fromSlotName, toSlotName}) {
            if(!sourceTargetKey){
                return;
            }
            const sourceTargetInstance = state
                .instances
                .get(sourceTargetKey);
            const sourceParentInstance = state
                .instances
                .get(sourceTargetInstance.parentInstanceKey);
            const targetParentInstance = state
                .instances
                .get(targetParentKey);
            debugger;
            if (sourceTargetInstance.parentInstanceKey !== targetParentKey) { // 跨越层级
                // 修改拖拽元素的 parentMapUniqueKey
                sourceTargetInstance.parentInstanceKey = targetParentKey;

                // 拖拽目标添加 instance
                if (!targetParentInstance
                    .slots[toSlotName]) {
                    targetParentInstance
                        .slots[toSlotName] = [];
                }
                targetParentInstance
                    .slots[toSlotName]
                    .splice(targetIndex, 0, sourceTargetKey);

                // 拖拽源删除 instance
                sourceParentInstance.slots[fromSlotName] = sourceParentInstance
                    .slots[fromSlotName]
                    .filter(childKey => childKey !== sourceTargetKey);
                this.commit('viewport/dealCurLevelAndTargetLevel', {
                    currentInstance: sourceTargetKey,
                    targetInstance: targetParentKey
                });
            } else { // 同层级
                this.commit('viewport/horizontalMoveInstance', {
                    parentKey: targetParentKey,
                    beforeIndex: sourceParentInstance
                        .slots[fromSlotName]
                        .findIndex(childKey => childKey === sourceTargetKey),
                    afterIndex: targetIndex
                });
            }
            // 触发 VueX 对map的检查
            state.instances = new Map(state.instances);
        },
        /**
         * 同层级拖拽，不需要主动调用，直接调用 moveInstance 即可
         */
        horizontalMoveInstance (state, {parentKey, beforeIndex, afterIndex, slotName}) {
            const parentInstance = state
                .instances
                .get(parentKey);
            if (beforeIndex < afterIndex) {
                // 从左到右
                for (let index = beforeIndex; index < afterIndex; index++) {
                    const beforeUniqueKey = parentInstance.slots[slotName][index];
                    const afterUniqueKey = parentInstance.slots[slotName][index + 1];
                    parentInstance.slots[slotName][index] = afterUniqueKey;
                    parentInstance.slots[slotName][index + 1] = beforeUniqueKey;
                }
            } else {
                // 从右到左
                for (let index = beforeIndex; index > afterIndex; index--) {
                    const beforeUniqueKey = parentInstance.slots[slotName][index];
                    const afterUniqueKey = parentInstance.slots[slotName][index - 1];
                    parentInstance.slots[slotName][index] = afterUniqueKey;
                    parentInstance.slots[slotName][index - 1] = beforeUniqueKey;
                }
            }
            // 触发 VueX 对map的检查
            state.instances = new Map(state.instances);
        },
        setDragInfo (state, {mapUniqueKey, index}) {
            const newInfo = state.currentDragInfo.info;
            newInfo.targetInstanceKey = mapUniqueKey;
            newInfo.targetIndex = index;
        },
        registerOuterDrag (state, dragParentDom) {
            // 上次拖拽的位置

            let lastDragStartIndex = -1;
            Sortable.create(dragParentDom, {
                animation: 50,
                // 放在一个组里,可以跨组拖拽
                group: {
                    name: 'gaea-container',
                    pull: 'clone',
                    put: false
                },
                sort: false,
                delay: 0,
                filter:".collapse-header",
                // draggable:".isDrag",
                onStart: (event) => {
                    lastDragStartIndex = event.oldIndex;
                    if (event.item.dataset.source) {
                        // this.startDrag({     type: 'combo',     dragStartParentElement:
                        // dragParentDom,     dragStartIndex: event.oldIndex ,     comboInfo: { source:
                        // event.item.dataset.source     } })
                    } else if (event.item.dataset.gaeaKey) {
                        state.dragStartDataReady = false;
                        this.commit('viewport/startDrag', {
                            type: 'new',
                            dragStartParentDom: dragParentDom,
                            dragStartIndex: event.oldIndex,
                            info: {
                                gaeaKey: event.item.dataset.gaeaKey,
                                props: event.item.dataset.props,
                                preGaeaKey: event.item.dataset.preGaeaKey
                            }
                        });
                        // 开始拖拽完毕
                        state.dragStartDataReady = true;
                    }
                },
                onEnd: (event) => {
                    this.commit('viewport/endDrag');
                    // 因为是 clone 方式, 拖拽成功的话元素会重复, 没成功拖拽会被添加到末尾 所以先移除 clone 的元素（吐槽下, 拖走的才是真的, 留下的才是
                    // clone 的） 有 parentNode, 说明拖拽完毕还是没有被清除, 说明被拖走了, 因为如果没真正拖动成功, clone 元素会被删除
                    if (event.clone.parentNode) {
                        //note 如果拖走了,把已经拖走的元素删除,只显示转换后的元素
                        event.item.parentNode.removeChild(event.item);
                    } else {
                        //note 没拖走, 只是晃了一下, 不用管了
                    }


                }
            });
        },
        //note 创建元素和实例
        createInstanceEle(newInfo){
            this.commit('viewport/addInstance', {
                gaeaKey: newInfo.gaeaKey,
                parentInstanceKey:newInfo.parentInstanceKey,
                indexPosition: 0,
                preGaeaKey: newInfo.preGaeaKey,
                slotName:newInfo.slotName,

                cb: newInstanceKey => {
                    if (onDragAdd) {
                        // e --event parentInstanceKey  --父instanceKey gaeaKey  -->component type
                        // newInstanceKey  -> new instace Key
                        onDragAdd.call(this, null,{}, newInfo.parentInstanceKey, newInfo.gaeaKey, newInstanceKey, newInfo.slotName);
                    }
                }
            });
        },
        registerInnerDrag (state, {
            parentInstanceKey,
            dragParentDom,
            params,
            groupName = 'gaea-container',
            onDragAdd
        }) {

            const instance = state
                .instances
                .get(parentInstanceKey);
            let oldObj={

                animation: 50,
                // 放在一个组里,可以跨组拖拽
                group: {
                    name: groupName,
                    pull: true,
                    put: true
                },
                onStart: (event) => {
                    let slotName = event.from.dataset.slotName;
                    let curInstance= state
                        .instances
                        .get(instance.slots[slotName][event.oldIndex]);
                    let newObj=Object.assign({}, curInstance);
                    state.saveDragObj=newObj;

                    this.commit('viewport/startDrag', {
                        type: 'viewport',
                        dragStartParentDom: dragParentDom,
                        dragStartIndex: event.oldIndex,
                        info: {
                            instanceKey: instance.slots[slotName][event.oldIndex]
                        }
                    });
                },
                onEnd: (event) => {
                    this.commit('viewport/endDrag');

                    // 在 viewport 中元素拖拽完毕后, 为了防止 outer-move-box 在原来位置留下残影, 先隐藏掉
                    this.commit('viewport/setCurrentHoverInstanceKey', null);
                },
                onAdd: (event) => {
                    // 如果数据还没有 ready，很尴尬，什么都没有发生 此时一定是 new，不用担心 dom 节点脏掉
                    if (!state.dragStartDataReady) {
                        return;
                    }
                    switch (state.currentDragInfo.type) {
                        case 'new':
                            // 是新拖进来的, 不用管, 因为工具栏会把它收回去 为什么不删掉? 因为这个元素不论是不是 clone, 都被移过来了, 不还回去 react 在更新
                            // dom 时会无法找到
                            const newInfo = state.currentDragInfo.info;
                            const slotName = event.to.dataset.slotName;

                            this.commit('viewport/addInstance', {
                                gaeaKey: newInfo.gaeaKey,
                                parentInstanceKey,
                                indexPosition: event.newIndex,
                                preGaeaKey: newInfo.preGaeaKey,
                                slotName,
                                cb: newInstanceKey => {
                                    if (onDragAdd) {
                                        // e --event parentInstanceKey  --父instanceKey gaeaKey  -->component type
                                        // newInstanceKey  -> new instace Key
                                        onDragAdd.call(this, null,function(){return {}}, parentInstanceKey, newInfo.gaeaKey, newInstanceKey, slotName);
                                    }
                                }
                            });
                            break;
                        case 'viewport':
                            // 设置新增时拖拽源信息
                            this.commit('viewport/setDragInfo', {
                                mapUniqueKey: parentInstanceKey,
                                index: event.newIndex
                            });
                            break;
                        case 'combo':
                            // this.addComboComponentBySource(mapUniqueKey,
                            // this.viewport.currentDragComponentInfo.comboInfo.source, event.newIndex )
                            // TODO 发布新增组合事件 this.props.viewport.saveOperate({     type: 'addCombo',
                            // mapUniqueKey,     addCombo: {         parentMapUniqueKey:
                            // this.props.mapUniqueKey,         index: event.newIndex , componentInfo:
                            // component     } })
                            break;
                    }
                },
                onUpdate: (event) => {
                    let slotName = event.from.dataset.slotName;
                    //  同一个父级下子元素交换父级 // 取消 srotable 对 dom 的修改, 让元素回到最初的位置即可复原
                    this.commit('viewport/horizontalMoveInstance', {
                        parentKey: parentInstanceKey,
                        beforeIndex: event.oldIndex,
                        afterIndex: event.newIndex,
                        slotName
                    });
                },
                onRemove: (event) => {
                    // onEnd 在其之后执行，会清除拖拽目标的信息 减少了一个子元素，一定是发生在 viewport 区域元素发生跨父级拖拽时
                    let toSlotName = event.to.dataset.slotName;
                    let fromSlotName = event.from.dataset.slotName;
                    const dragTargetKey = state
                        .instances
                        .get(parentInstanceKey)
                        .slots[toSlotName][state.currentDragInfo.dragStartIndex];
                    const dragViewportInfo = state.currentDragInfo.info;

                    let curInstance= state
                        .instances
                        .get(dragTargetKey);
                    // if(typeof(curInstance)!="undefined"&&curInstance.vm.level>2){
                    if(typeof(curInstance)=="undefined"){
                        return;
                    }
                    this.commit('viewport/moveInstance', {
                        sourceTargetKey: dragTargetKey,
                        targetParentKey: dragViewportInfo.targetInstanceKey,
                        targetIndex: dragViewportInfo.targetIndex,
                        fromSlotName,
                        toSlotName
                    });
                }
            }
            let newObj = Object.assign(params, oldObj);
            Sortable.create(dragParentDom,newObj);


            //note 删除当前拖拽的组件
            this.commit('viewport/dealLevel', state);
        },
        getInstancePath (state, {instanceKey, cb}) {
            const finderPath = [state.currentEditInstanceKey];
            if (state.currentEditInstanceKey === null) {
                if (cb) {
                    let result = [];
                    cb(result);
                }
                return;
            }
            let instance = state
                .instances
                .get(state.currentEditInstanceKey);
            // 如果已经是根元素, 直接返回空数组
            if (instance.parentInstanceKey === null) {
                if (cb) {
                    let result = [state.rootInstanceKey];
                    cb(result);
                }
                return;
            }

            // 直到父级是根元素为止
            while (state.instances.get(instance.parentInstanceKey).parentInstanceKey !== null) {
                finderPath.unshift(instance.parentInstanceKey);
                instance = state
                    .instances
                    .get(instance.parentInstanceKey);
            }
            finderPath.unshift(state.rootInstanceKey);
            if (cb) {
                cb(finderPath);
            }
        },
        setCurrentEditInstanceKey (state, instanceKey) {
            // 如果和当前正在编辑元素相同，不做操作
            if (state.currentEditInstanceKey === instanceKey) {
                return;
            }
            // 修改 mapUniqueKey
            state.currentEditInstanceKey = instanceKey;
        },
        setPagePriviewData(state,pageObjData){
            //设置页面数据
            state.pagePriviewData = pageObjData;
        }
    },
    actions: {}
};
//note 自定义mutation
let selfMutation={
    //note 显示弹层


    updateInputData(state){
        let currentInstance= state.instances.get(state.currentEditInstanceKey);

        for(let prop in currentInstance.data.property){
            currentInstance.data.property[prop]="哈哈哈";
        }
        for(let prop in currentInstance.data.property){
            currentInstance.data.property[prop]="哈哈哈";
        }
        state.instances = new Map(state.instances);
        /**/
        /* this.commit('viewport/setInstanceProps', {
                                      vm: currentInstance.vm,
                                      key: 'property',
                                      value: {
                                          label: '哈哈哈'
                                      }
                                  });*/
    },
    //NOTE 修改全局值
    updateData(state,element){


        /* state.instances = new Map(state.instances);
         return;*/

        let allInstance= state.instances;
        let localInstance= state.localInstances;
        localInstance= [...localInstance];
        allInstance= [...allInstance];


        localInstance.splice(0,0,allInstance[0])

        let _this=this;
        for(let key=0;key<localInstance.length;key++){
            let keyName=localInstance[key][0];
            let val=localInstance[key][1];
            if(keyName!="gaea_instance_1"){
                //给我们的实例赋值
                //      allInstance[key][1].data=new Object(localInstance[key][1].data);
                // allInstance[key][1].data.property.label="哈哈哈";

                // allInstance[key][1].data.property=localInstance[key][1].data.property;
                for(let prop in allInstance[key][1].data){
                    allInstance[key][1].data[prop]=localInstance[key][1].data[prop];
                }
                /*for(let prop in localInstance[key][1].data.property){
                    allInstance[key][1].data.property[prop]=localInstance[key][1].data[prop];
                }*/
            }
        }
        state.instances=allInstance;
        state.instances = new Map(state.instances);


        /* for(let key=0;key<localInstance.length;key++){
             let keyName=localInstance[key][0];
             let val=localInstance[key][1];
             if(keyName!="gaea_instance_1"){
                 //给我们的实例赋值
                 //  allInstance[key][1].data.property.label="哈哈哈";
                 allInstance[key][1].vm.$forceUpdate();
             }
         }*/

        /*currentInstance.data.property.label="哈哈哈";
        state.instances = new Map(state.instances);*/
        /*this.commit('viewport/setInstanceProps', {
                                     vm: currentInstance.vm,
                                     key: 'property',
                                     value: {
                                         label: '哈哈哈'
                                     }
                                 });*/

    },
    //NOTE 存储元素 localInstances
    updateInstances(state,element){

        state.instances=state.localInstances;
        state.instances = new Map(state.instances);
    },
    //NOTE 存储元素 localInstances
    saveLocalElement(state,element){
        state.localElement=element;
    },
    //NOTE 存储实例 localInstances
    saveLocalInstances(state,instances){
        debugger;
        let localInstances= [...state.instances];
        localInstances.splice(0,1);
        for(let key=0;key<localInstances.length;key++){
            delete localInstances[key][1].vm;
            // val.data.property.value=val.data.property.label;
        }
        console.log(localInstances); // [ 1, 2, 3 ]  openModalTitle
        let templateJSON=JSON.stringify(localInstances);
        //  eventbus.$emit("openModalTitle",localInstances);
        let toServerParams = {
            itemNo:localStorage.itemNo,
            itemStep:localStorage.itemStep,
            jsonType:1,
            jsonStr:templateJSON
        };
        Service.saveTemplateOrJsonToServer(toServerParams).then((res) => {

            //note 刚才保存完了json，现在要保持模板
            eventbus.$emit("openSuccDialog");
        })
        //note 发送网络请求,把保存的模板提交给后台
        state.localInstances=localInstances;
    },
    /*
     Note 增加删除单选内容选项
    */
    delRadioEventArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.radioArr) {
            currentInstance.data.radioArr = [];
        }
        // 为事件新增一项
        currentInstance.data.radioArr.splice(childObj.index,1);
        state.instances = new Map(state.instances);
    },
    addRadioEventArr(state){
        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.radioArr) {
            currentInstance.data.radioArr = [];
        }
        // 为事件新增一项
        currentInstance.data.radioArr.push({
            label: "",
            value: "",
            extend: {
                size: '',
                disabled: false,
            }
        });
        state.instances = new Map(state.instances);
    },
    /*
 Note 删除增加校验
*/
    delValidateEventArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.validateArr) {
            currentInstance.data.validateArr = [];
        }
        // 为事件新增一项
        currentInstance.data.validateArr.splice(childObj.index,1);
        state.instances = new Map(state.instances);
    },
    addValidateEventArr(state){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.validateArr) {
            currentInstance.data.validateArr = [];
        }
        // 为事件新增一项
        currentInstance.data.validateArr.push({
            required: false,
            message: "",
            type: "",
            trigger: "",
            dot: "",
            range: 0,
            unit: "",
        });
        state.instances = new Map(state.instances);
    },
    /*
   Note 步骤增加 删除
 */
    addLinkItemsEventArr(state){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.items) {
            currentInstance.data.items = [];
        }
        // 为事件新增一项
        currentInstance.data.items.push({
            label: "",
            prop: "",
            isShow: "",
        });
        state.instances = new Map(state.instances);
    },
    addLinkOptionsEventArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.items[childObj.index].options) {
            currentInstance.data.items[childObj.index].options = [];
        }
        // 为事件新增一项
        currentInstance.data.items[childObj.index].options.push({
            name: "",
            value: 0,
            isSelected:false
        });
        state.instances = new Map(state.instances);
    },
    addLinkExactEventArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.items[childObj.index].exact) {
            currentInstance.data.items[childObj.index].exact = [];
        }
        // 为事件新增一项
        currentInstance.data.items[childObj.index].exact.push({
            type: "",
        });
        state.instances = new Map(state.instances);
    },
    delLinkItemsEventArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.items) {
            currentInstance.data.items = [];
        }
        // 为事件新增一项
        currentInstance.data.items.splice(childObj.index,1);
        state.instances = new Map(state.instances);
    },
    delLinkExactEventChildArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.items) {
            currentInstance.data.items = [];
        }
        // 为事件新增一项
        currentInstance.data.items[childObj.parentIndex].exact.splice(childObj.index,1);
        state.instances = new Map(state.instances);
    },
    delLinkOptionsEventChildArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.linkageChangeValue) {
            currentInstance.data.linkageChangeValue = [];
        }
        // 为事件新增一项
        currentInstance.data.items[childObj.parentIndex].options.splice(childObj.index,1);
        state.instances = new Map(state.instances);
    },
    /*
    Note 步骤增加 删除
  */
    delStepsEventArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.steps) {
            currentInstance.data.steps = [];
        }
        // 为事件新增一项
        currentInstance.data.steps.splice(childObj.index,1);
        state.instances = new Map(state.instances);
    },
    addStepsEventArr(state){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.steps) {
            currentInstance.data.steps = [];
        }
        // 为事件新增一项
        currentInstance.data.steps.push({
            title: ""
        });
        state.instances = new Map(state.instances);
    },
    /*
     Note 联动增加 删除
     */
    delLinkEventArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.linkageChangeValue) {
            currentInstance.data.linkageChangeValue = [];
        }
        // 为事件新增一项
        currentInstance.data.linkageChangeValue.splice(childObj.index,1);
        state.instances = new Map(state.instances);
    },
    delLinkEventChildArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.linkageChangeValue) {
            currentInstance.data.linkageChangeValue = [];
        }
        // 为事件新增一项
        currentInstance.data.linkageChangeValue[childObj.parentIndex].values.splice(childObj.index,1);
        state.instances = new Map(state.instances);
    },
    addLinkEventChildArr(state,childObj){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.linkageChangeValue) {
            currentInstance.data.linkageChangeValue = [];
        }
        // 为事件新增一项
        currentInstance.data.linkageChangeValue[childObj.index].values=[];
        currentInstance.data.linkageChangeValue[childObj.index].values.push({
            dependValue: "",
            changeValue: "",
            changeParam:"",
            changeAttrs: {
                "readonly": false,
                "clearable": false
            }
        });
        state.instances = new Map(state.instances);
    },
    addLinkEventArr(state){

        let currentInstance= state.instances.get(state.currentEditInstanceKey);
        if (!currentInstance.data.linkageChangeValue) {
            currentInstance.data.linkageChangeValue = [];
        }
        // 为事件新增一项
        currentInstance.data.linkageChangeValue.push({
            originKey: "",
            changeKey: "",
            type: "",
            isValue:false,
            noChangeExist:false
        });
        state.instances = new Map(state.instances);
    },
    //note 删除
    deleteLevel1Data(state){

        //NOTE 获取所有级别为1的（排除根节点），全部删除
        let localInstances= [...state.instances];
        let myArr=localInstances.splice(0,1);
        myArr[0][1].slots.default=[];
        state.instances = new Map(myArr);
        /* let level1Arr=[];//note 定义级别为1的数组
         for(let key=0;key<localInstances.length;key++){
             //if(localInstances[key][1].data.level==1){
                 level1Arr.push(localInstances[key][0]);
             //}
         }
         //note 开始删除
         for(let key=0;key<level1Arr.length;key++){
             this.commit('viewport/deleteData',level1Arr[key]);
         }*/
        //note 初始化模板的值

        eventbus.$emit("initTemplateInstance");
    },
    deleteData(state,currentKey){

        let currentChild= state.instanceDoms.get(currentKey);
        let parent = currentChild.parentNode;
        parent.removeChild(currentChild);
        state.instanceDoms.delete(currentKey);
        //取出当前删除元素的父元素,并在父元素中删除对应的元素
        let parentInstanceKey =  state.instances.get(currentKey).parentInstanceKey;
        if(parentInstanceKey){
            let childArr = state.instances.get(parentInstanceKey).slots.default;
            _.pullAll(childArr,[currentKey]);
        }
        let instancesObj = state.instances.get(currentKey);
        //连级删除自身和子元素
        this.commit('viewport/deleteAll',{instancesObj:instancesObj,delObj:currentKey});
        state.instances = new Map(state.instances);
        // console.log(state);
    },
    deleteAll(state,data){
        if(data.instancesObj.slots.default&&data.instancesObj.slots.default.length>0){
            data.instancesObj.slots.default.forEach((item)=>{
                let instancesObj = state.instances.get(item);
                this.commit('viewport/deleteAll',{instancesObj:instancesObj,delObj:item});
            });
        }
        state.instances.delete(data.delObj);
    }
}
let newObj=Object.assign(viewport.mutations, selfMutation);
viewport.mutations=newObj;
export default viewport;
